from __future__ import print_function, division

"""
Power-Grid Dynamical Network Simulation.
=======================================================================

This function simulates a Power-Grid (PG) dynamical network.
"""
def PG_steady_state(N, P, K):
    """ Find steady state solution for given network
        
    The default parameters are for the first example in the results section.
    """
    import numpy as np
    from scipy.optimize import minimize
    omega_ss = np.zeros((N,))
    theta_ss = np.zeros((N,))
    def cost_function(theta_ss):
        cost = []
        for i in range(N):
            cost_i = P[i]
            for j in range(N):
                cost_i += K[i][j]*np.sin(theta_ss[j] - theta_ss[i])
            cost.append(cost_i)
        return np.sum(np.array(cost)**2)
    res = minimize(cost_function, x0 = theta_ss, method='Nelder-Mead', tol=1e-6)
    theta_ss = res.x
    return theta_ss, omega_ss
    
def PG_flows(theta, omega, K, N):
    """ Calculates flows from theta and omega of steady state
    """
    import numpy as np
    F = np.zeros((N,N))
    for i in range(N):
        for j in range(N):
            F[i][j] = K[i][j]*np.sin(theta[j] - theta[i])
    return F

def get_edge_list(A):
    """ From adjacency matrix go to list of edges.
    """
    import numpy as np
    A_ut = np.triu(A)
    E = []
    for i in range(len(A_ut)):
        for j in range(len(A_ut[i])):
            if A_ut[i][j] != 0:
                E.append([i,j])
    return np.array(E)
    
def PG_network(A, t, V_gen, V_con, K_0 = 1.63, P_gen = 1.5, P_con = 1, alpha = 0.6, I_0 = 1,
                  damping = 0.5, e_cut = (2,4)):
    """This function simulates and returns a power grids network through the adjacency matrix over time t.

    Args:
       t (array): time array

    Kwargs:
       plotting (bool): Plotting for user interpretation. defaut is False.

    Returns:
       (matrix): A, an n by n matrix A over time t.

    """
    
    import networkx as nx
    import numpy as np

    A_0 = np.copy(A) #use copy of adjacency matrix for plotting all the flows
    E = get_edge_list(A)

    def swing_equation(omega, theta):
        d_theta, d_omega = np.zeros((N,)), np.zeros((N,))
        for i in range(N):
            d_theta[i] = omega[i]
            d_omega[i] = P[i] - G[i]*omega[i]
            for j in range(N):
                d_omega[i] = d_omega[i] + K[i][j]*np.sin(theta[j] - theta[i])
                #I believe the authors used A[i][j] here instead of K[i][j] as they state in their equations.
            d_omega[i] = d_omega[i]/I[i]
        return d_theta, d_omega
    
    
    G = nx.convert_matrix.from_numpy_matrix(A)
    N = G.number_of_nodes()
    
    P = np.zeros((N,)) 
    P[V_gen] = P_gen
    P[V_con] = -P_con
    
    K = K_0*A
    I = np.zeros((N,)) + I_0 #inertia of power generators
    G = np.zeros((N,)) + damping #damping of rotary machine
    C = alpha*K
    
    
    
    #get new steady state values
    
    
    theta, omega = PG_steady_state(N, P, K)
    Fs = []
    dt = t[1]-t[0]
    for t_i in t:
        if t_i > 1: # remove line 4-2 at 1 second
            K[e_cut[0]][e_cut[1]], K[e_cut[1]][e_cut[0]] = 0, 0
            
        d_theta, d_omega = swing_equation(omega, theta)
        theta, omega = theta+d_theta*dt, omega+d_omega*dt
        F = PG_flows(theta, omega, K, N)
        
        for e in E:
            if np.abs(F[e[0],e[1]]) > C[e[0],e[1]]: # if flow is greater than line capacity
                K[e[0]][e[1]], K[e[1]][e[0]] = 0, 0
            
        flows = []
        for e in E:
            flows.append(np.abs(F[e[0], e[1]]))
        Fs.append(flows)
        
    Fs = np.array(Fs).T  
    
    return Fs, E


# In[ ]:


if __name__ == '__main__':
    #from dynamical_networks.simulate.PG_network import PG_network
    
    import matplotlib.pyplot as plt
    import numpy as np
    
    t = np.linspace(0,12, 5000) #the time which the simulation will be solved for.
    P_gen = 1.5 #the amount of power generated by the generators.
    P_con = 1 #the power consumed by the consumers.
    K_0 = 1.63 #the capacitance of the lines
    I_0 = 1 #inertia of the power generators and consumers
    damping = 0.5 #the damping in the power oscillations or losses in the transmission of power.
    alpha = 0.6 #a factor of safety for for the capacity of the power lines
    V_gen = np.array([1, 4]) #vertices indices of generates
    V_con = np.array([0, 2, 3]) #vertices indices of consumers
    A = np.array([[0, 1, 1, 0, 1],
                  [1, 0, 1, 1, 0],
                  [1, 1, 0, 1, 0],
                  [0, 1, 1, 0, 1],
                  [1, 0, 0, 1, 0]])
    
    
    
    Fs, E = PG_network(A, t, V_gen, V_con, K_0, P_gen, P_con, alpha, I_0, damping, e_cut = (1,3))
    
    
    #---------------PLOTTING-------------------
    
    plt.figure(figsize = (10,5))
    TextSize = 35
    for i in range(len(Fs)):
        plt.plot(t, Fs[i], label = '$('+str(E[i][0])+','+str(E[i][1])+')$')
    plt.plot([0,max(t)], [alpha*K_0, alpha*K_0], 'k--')
    
    plt.grid()
    plt.xlim(0,max(t))
    plt.ylim(0,)
    plt.xticks(size = TextSize)
    plt.yticks(size = TextSize)
    plt.xlabel(r'$t$', size = TextSize)
    plt.ylabel(r'$|F_{i,j}|$', size = TextSize)
    plt.legend(loc = 'upper right', fontsize = TextSize-12, ncol = 1)
    plt.show()
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    


"""
Evolutionary homology calculation for dynamical networks.
=======================================================================

This functions calculates the evelutionary homology of a dynamical networks adjacency matrix.
"""

def EveHom(A):
    """This function takes the time varying adjacency matrix and returns the evolutionary homology.

    Args:
       A (array): Time varying adjacency matrix.

    Kwargs:
       plotting (bool): Plotting for user interpretation. defaut is False.

    Returns:
       (array): Statistic array over time.

    """

    import numpy as np
    
    #-------------------Run input data tests--------------------------
    if type(A) != np.ndarray:
        # the data type should be a numpy array of 2d arrays
        raise TypeError("Input must be numpy array.")
    if type(A[0][0]) != np.ndarray:
        # the size of each adjacency matrix should not be empty
        raise TypeError("Input must be numpy array of 2D (nxn) arrays. i.e. Nx(nxn) array.")
        
    statistic = []
    for a in A:
        stat = np.nanmax(a)
        statistic.append(stat)
    
    return np.array(statistic)


# In[ ]:


if __name__ == '__main__':
    
    from dynamical_networks.analysis.evolutionary_homology import EveHom
    from dynamical_networks.simulate.PG_network import PG_network
    A = PG_network()
    S = EveHom(A)






# -*- coding: utf-8 -*-
"""
Created on Tue Mar  9 11:03:26 2021

@author: myersau3
"""

def check_boundaries(x_new_i, y_new_i, B):
    x_width, y_width = B[0][1] - B[0][0], B[1][1] - B[1][0]
    if x_new_i > B[0][1]:
        x_new_i = x_new_i-x_width
    if x_new_i < B[0][0]:
        x_new_i = x_new_i+x_width
    if y_new_i > B[1][1]:
        y_new_i = y_new_i-y_width
    if y_new_i < B[1][0]:
        y_new_i = y_new_i+y_width
    return x_new_i, y_new_i
        
def vicsek_model(t, initial_conditions, velocity, noise_std, eps, B):
    dt = np.mean(np.diff(t))
    x, y = initial_conditions.T[0], initial_conditions.T[1]
    theta = initial_conditions.T[2]
    I = np.arange(len(x))
    dx, dy = v*dt*np.cos(theta), v*dt*np.sin(theta)
    pos = []
    for t_i in t:
        pos.append(np.array([x, y, theta]).T)
        x_new = []
        y_new = []
        theta_new = []
        for i in range(len(initial_conditions)):
            x_i, y_i, theta_i = x[i], y[i], theta[i]
            d_i = np.sqrt((x-x_i)**2 + (y-y_i)**2)
            neighbors = I[d_i<eps]
            theta_ave = np.mean(theta[neighbors])
            theta_new_i = theta_ave + np.random.normal(0,noise_std,1)[0]
            dx, dy = v*dt*np.cos(theta_i), v*dt*np.sin(theta_i)
            x_new_i, y_new_i = x_i+dx, y_i+dy
            x_new_i, y_new_i = check_boundaries(x_new_i, y_new_i, B)
            
            x_new.append(x_new_i)
            y_new.append(y_new_i)
            theta_new.append(theta_new_i)
        x, y, theta = np.array(x_new), np.array(y_new), np.array(theta_new)
    pos = np.array(pos)
    
    plotting = True
    if plotting == True:
        for i in np.arange(0,len(pos), 2):
            x, y = pos[i].T[0], pos[i].T[1]
            plt.figure(figsize = (7,7))
            plt.plot(x,y, 'k.')
            lim = 1
            plt.xlim(-0,lim)
            plt.ylim(-0,lim)
            plt.show()     
        
    return pos
    
    
# In[ ]: 

if __name__ == "__main__": 
        
    import numpy as np
    import matplotlib.pyplot as plt
    
    n = 100
    v = 10
    dt = 0.01
    x = np.random.rand(n,1).T[0]
    y = np.random.rand(n,1).T[0]
    theta = 2*np.pi*np.random.rand(n,1).T[0]
    
    
    nu = 0.1
    IC = np.array([x, y, theta]).T
    t = np.linspace(0,3,2000)
    B = [[-0,1],[-0,1]]
    
    vicsek_model(t = t, initial_conditions = IC, velocity = v, 
                 noise_std = nu, eps = 0.05, B = B)
    
    
    
    
    
    
    
    
    
    
    
    
    


# -*- coding: utf-8 -*-
"""
Created on Wed Mar 10 14:02:55 2021

@author: myersau3
"""

import os
import sys
sys.path.insert(0, os.path.join(os.path.dirname(__file__),'..'))
from analysis import statistics # centrality, capacity, node_degree_distribution, betweenness

import pytest
import numpy as np 


def test_statistics_list_for_A():
    # test to make sure a list of lists for A does not work
    A = [[[0,0],[1,1]], [[0,0],[1,1]]]
    with pytest.raises(TypeError) as excinfo:
        statistics.centrality(A)
    with pytest.raises(TypeError) as excinfo:
        statistics.betweenness(A)
    with pytest.raises(TypeError) as excinfo:
        statistics.capacity(A)
    with pytest.raises(TypeError) as excinfo:
        statistics.node_degree_distribution(A)
        
        
def test_statistics_for_size_of_A():
    # test to make sure a list of lists for A does not work
    A = np.array([[0,0],[1,1]])
    with pytest.raises(TypeError) as excinfo:
        statistics.centrality(A)
    with pytest.raises(TypeError) as excinfo:
        statistics.betweenness(A)
    with pytest.raises(TypeError) as excinfo:
        statistics.capacity(A)
    with pytest.raises(TypeError) as excinfo:
        statistics.node_degree_distribution(A)
        
def test_statistics_correct_output():
    # test if function returns numpy array for good input file.
    A = np.array([[[0,0],[1,1]], [[0,0],[1,1]]])
    S = statistics.centrality(A)
    assert type(S) == np.ndarray
    S = statistics.capacity(A)
    assert type(S) == np.ndarray
    S = statistics.node_degree_distribution(A)
    assert type(S) == np.ndarray
    S = statistics.betweenness(A)
    assert type(S) == np.ndarray


# -*- coding: utf-8 -*-
"""
Created on Wed Mar 10 14:02:55 2021

@author: myersau3
"""

import os
import sys
sys.path.insert(0, os.path.join(os.path.dirname(__file__),'..'))
from analysis import statistics # centrality, capacity, node_degree_distribution, betweenness

import pytest
import numpy as np 





"""
Manufacturer-Supplier Dynamical Network Simulation.
=======================================================================

This function simulates a Manufacturer-Supplier (MS) dynamical network.
"""


def MS_network(A_MS, M_arr, S_arr, make_gif = False, make_graph_plot = False):
    """This function takes system parameters to develop a time dependent adjacency matrix A(t).

    Args:
       A_MS (array):  Unweighted adjacency matrix for network.
       M_arr (array): Throughput array for manufacturers.
       S_arr (array): Throughput array for suppliers.

    Kwargs:
       plotting (bool): Plotting for user interpretation. defaut is False.

    Returns:
       (matrix): A, an n by n matrix A over time t.

    """
    import numpy as np
    import networkx as nx
    import matplotlib.pyplot as plt
    E = []
    num_man = len(A_MS)
    num_sup = len(A_MS[0])
    for m in range(num_man):
        for s in range(num_sup):
            if A_MS[m][s] != 0:
                E.append([m,s+num_man])
        
    V_M = np.arange(num_man).astype(int) 
    V_S = np.arange(num_man, num_man+num_sup).astype(int) 
    n = num_man+num_sup
    G = nx.Graph()
    G.add_nodes_from(V_M, bipartite=0)
    G.add_nodes_from(V_S, bipartite=1)
    G.add_edges_from(E)
    
    As = []
    for t in np.arange(0,len(M_arr.T[0])):
        A = np.zeros((n,n))
        for e in E:
            w1 = M_arr.T[e[0]][t]
            w2 = S_arr.T[e[1]-num_man][t]
            e_ij = np.min([w1, w2])
            A[e[0], e[1]] = e_ij
        A = A+A.T
        As.append(A)
      
        
    
    if make_gif == True:
        top = nx.bipartite.sets(G)[0]
        pos = nx.bipartite_layout(G, top)
        colors = ['lightgreen'] * num_man + ['lightblue'] * num_sup
        labeldict = {}
        for i in range(num_man): 
            labeldict[i] = r'$M_{'+str(i)+'}$'
        for i in range(num_sup): 
            j = i+num_man
            labeldict[j] = r'$S_{'+str(i)+'}$'
            
        frame_count = 0
        for j in np.arange(0,len(M_arr.T[0]),int(len(M_arr.T[0])/100)):
            node_sizes = []
            M_max = np.amax(M_arr)
            S_max = np.amax(S_arr)
            for i in range(num_man): 
                node_sizes.append(2000*M_arr.T[i][j]/M_max)
            for i in range(num_sup): 
                node_sizes.append(2000*S_arr.T[i][j]/S_max)
                
            weights = []
            for e in E:
                w1 = M_arr.T[e[0]][j]/M_max
                w2 = S_arr.T[e[1]-num_man][j]/S_max
                w = np.min([w1, w2])
                weights.append(4*w) 
                
            plt.figure(figsize = (4,3))
            nx.draw(G, pos=pos, with_labels=False, labels = labeldict, 
                    node_color=colors, node_size = node_sizes, width = weights)
            axis = plt.gca()
            axis.set_xlim([1.25*x for x in axis.get_xlim()])
            axis.set_ylim([1.25*y for y in axis.get_ylim()])
            plt.text(x = -0.32, y = -0.15, s=' Dynamical \n Networks',
                     ma = 'center',
                     wrap = True, fontsize = 30, 
                     bbox=dict(boxstyle="round",ec=(0.0, 0.0, 0.0),fc=(0.85, 0.85, 0.85),))
            plt.savefig('C:\\Users\\myersau3.EGR\\Desktop\\python_png\\frames\\frame'+str(frame_count)+'.png', 
                     pad_inches = 0.30, dpi = 200)
            plt.show()
            frame_count = frame_count + 1
        
        import imageio
        images = []
        for i in range(int(frame_count)):
            images.append(imageio.imread('C:\\Users\\myersau3.EGR\\Desktop\\python_png\\frames\\frame'+str(i)+'.png'))
        for i in range(int(frame_count)):
            i = int(frame_count) - i - 1
    
            images.append(imageio.imread('C:\\Users\\myersau3.EGR\\Desktop\\python_png\\frames\\frame'+str(i)+'.png'))
        imageio.mimsave('C:\\Users\\myersau3.EGR\\Desktop\\python_png\\MS_network_gif.gif', images, fps = 50)



    
    if make_graph_plot == True:
        top = nx.bipartite.sets(G)[0]
        pos = nx.bipartite_layout(G, top)
        colors = ['lightgreen'] * num_man + ['lightblue'] * num_sup
        labeldict = {}
        for i in range(num_man): 
            labeldict[i] = r'$M_{'+str(i)+'}$'
        for i in range(num_sup): 
            j = i+num_man
            labeldict[j] = r'$S_{'+str(i)+'}$'
            
        j = 0
        node_sizes = []
        M_max = np.amax(M_arr)
        S_max = np.amax(S_arr)
        for i in range(num_man): 
            node_sizes.append(2000*M_arr.T[i][j]/M_max)
        for i in range(num_sup): 
            node_sizes.append(2000*S_arr.T[i][j]/S_max)
            
            
        weights = []
        for e in E:
            w1 = M_arr.T[e[0]][j]/M_max
            w2 = S_arr.T[e[1]-num_man][j]/S_max
            w = np.min([w1, w2])
            weights.append(4*w) 
            
            
        plt.figure(figsize = (4,3))
        nx.draw(G, pos=pos, with_labels=False, labels = labeldict, 
                node_color=colors, node_size = node_sizes, width = weights)
        axis = plt.gca()
        axis.set_xlim([1.25*x for x in axis.get_xlim()])
        axis.set_ylim([1.25*y for y in axis.get_ylim()])
        plt.show()
        
        
    return np.array(As)






def MS_simulation(t, parameters):
    """This function takes system parameters to develop a time dependent throughput simulation of manufacturers and suppliers.

    Args:
       parameter (float):  system parameter or parameters.
       t (array): time array for simulation.

    Kwargs:
       plotting (bool): Plotting for user interpretation. defaut is False.

    Returns:
       (M and S arrays): Arrays of t for the throughput of manufacturers and suppliers.

    """
    
    import numpy as np
    from scipy.integrate import odeint
    import odeintw as OIW
    
    
    #setting simulation time series parameters
    M, K_M, alpha_M, B_M1, B_M2, mu_M, S, K_S, alpha_S, B_S1, B_S2, mu_S, h, A_MS = parameters
    m, s = len(M), len(S)
    A_MS = np.array(A_MS)
    alpha_M_scale, alpha_S_scale = alpha_M, alpha_S
    delta = 0.1 #interaction strength modulation
    
    def competition_function(i, M, B_M1_i, B_M2_i):
        C_sum = 0
        for j in range(len(B_M1_i)):
            if j != i:
                C_sum = C_sum + (B_M1_i[j] + B_M2_i[j])*M[j]
        return M[i]*C_sum
    
    def mutualistic_manufacturer_interaction_function(i, M, S, h, A_MS, delta):
        gamma_0 = 1 #mutualistic strength level
        MI_sum = 0
        for k in range(len(A_MS[i])): #go through each supplier of of manufacturer
            y_ik = A_MS[i][k]
            N_i = np.sum(A_MS[i]) #number of interactions
            gamma_ik = y_ik*gamma_0/(N_i**delta)
            MI_sum = MI_sum + gamma_ik*S[k]
        return M[i]*MI_sum/(1+h*MI_sum)
    def mutualistic_supplier_interaction_function(i, M, S, h, A_MS, delta):
        AT_MS = A_MS.T
        gamma_0 = 1 #mutualistic strength level
        MI_sum = 0
        for k in range(len(AT_MS[i])): #go through each supplier of of manufacturer
            y_ik = AT_MS[i][k]
            N_i = np.sum(AT_MS[i]) #number of interactions
            gamma_ik = y_ik*gamma_0/(N_i**delta)
            MI_sum = MI_sum + gamma_ik*M[k]
        return S[i]*MI_sum/(1+h*MI_sum)
    
    def maximum_throughput_to_suplier_from_manufacturers(A_MS, M, i):
        A_M_to_S_i = A_MS.T[i] #find edges in network to find suppliers manufacturers
        max_M_to_S_i = 0 #begin at 0 possible sources from manufacturers
        for ind in range(len(A_M_to_S_i)):
            if A_M_to_S_i[ind] == 1: #if there is a connection
                max_M_to_S_i = max_M_to_S_i + M[ind] #add throughput of manufacturer to total
        return max_M_to_S_i
    
    def maximum_needed_of_suppliers_from_manufacturer(A_MS, S, i):
        A_S_from_M_i = A_MS[i] #find edges in network to find suppliers manufacturers connections
        max_S_need_from_M = 0 #begin at 0 possible need from manufactuers
        for ind in range(len(A_S_from_M_i)):
            if A_S_from_M_i[ind] == 1: #if there is a connection
                max_S_need_from_M = max_S_need_from_M + S[ind] #add throughput of manufacturer to total
        return max_S_need_from_M
    
    #defining simulation functions
    def dot_MS(M, K_M, alpha_M, B_M1, B_M2, mu_M,
               S, K_S, alpha_S, B_S1, B_S2, mu_S,
               h, A_MS, delta):
        
        #M is array of manufacturers
        #S is array of suppliers
        dM = [] #state space time derivatives for M and S
        for i in range(len(M)):
            M_i, K_M_i, alpha_M_i = M[i], K_M[i], alpha_M[i]
            B_M1_i, B_M2_i = B_M1[i], B_M2[i]
            mu_M_i = mu_M[i]
            K_M_from_S_i = maximum_needed_of_suppliers_from_manufacturer(A_MS, S, i)
            if K_M_from_S_i > K_M_i: 
                K_MS_i = K_M_i
            else:
                K_MS_i = K_M_from_S_i
            growth = M_i*(1-M_i/K_MS_i)
            internal_perturbation = alpha_M_i*M_i
            competition = competition_function(i, M, B_M1_i, B_M2_i)
            mutualistic_interaction = mutualistic_manufacturer_interaction_function(i, M, S, h, A_MS, delta)
            dM_i = growth - internal_perturbation - competition + mutualistic_interaction + mu_M_i
            dM.append(dM_i)
        
        dS = [] #state space time derivatives for M and S
        for i in range(len(S)):
            S_i, K_S_i, alpha_S_i = S[i], K_S[i], alpha_S[i]
            B_S1_i, B_S2_i = B_S1[i], B_S2[i]
            mu_S_i = mu_S[i]
            K_M_to_S_i = maximum_throughput_to_suplier_from_manufacturers(A_MS, M, i)
            if K_M_to_S_i > K_S_i: 
                K_MS_i = K_S_i
            else:
                K_MS_i = K_M_to_S_i
            growth = S_i*(1-S_i/K_MS_i)
            internal_perturbation = alpha_S_i*S_i
            competition = competition_function(i, S, B_S1_i, B_S2_i)
            mutualistic_interaction = mutualistic_supplier_interaction_function(i, M, S, h, A_MS, delta)
            dS_i = growth - internal_perturbation - competition + mutualistic_interaction + mu_S_i
            dS.append(dS_i)
            
        return np.array(dM), np.array(dS)
    
    
    dt = np.mean(np.diff(t))
    T = len(t)
    M_arr = [M]
    S_arr = [S]
    perturbation_CD, kill_CD = 0, 0
    alpha_M = np.random.rayleigh(0,m) #internal perturbation of manufacturer
    alpha_S = np.random.rayleigh(0,s) #internal perturbation of supplier
    for i in range(T):
        perturbation_CD = perturbation_CD+dt
        kill_CD = kill_CD+dt
        if perturbation_CD > 1:
            alpha_M = np.random.rayleigh(alpha_M_scale,m) #internal perturbation of manufacturer
            alpha_S = np.random.rayleigh(alpha_S_scale,s) #internal perturbation of supplier
            perturbation_CD = 0
        
        
        dM, dS = dot_MS(M, K_M, alpha_M, B_M1, B_M2, mu_M,S, K_S, alpha_S, B_S1, B_S2, mu_S, h, A_MS, delta)
        M, S = M+dM*dt, S+dS*dt
        
        M_arr.append(M)
        S_arr.append(S)
    
    return np.array(M_arr)[:-1], np.array(S_arr)[:-1]



# In[ ]:


if __name__ == '__main__':
    
    from MS_network import MS_simulation, MS_network
    import numpy as np
    import matplotlib.pyplot as plt
    
    
    #----------------------System Parameters and intial conditions------------------------
    M = np.zeros((4,))+0.1                    #current throughput of manufacturer
    S = np.zeros((3,))+0.1                     #current throughput of supplier 
    m = len(M)
    s = len(S)
    a = np.ones(s*m)
    edge_density = 0.5
    a[:int((m*s)*(1-edge_density))] = 0
    np.random.shuffle(a)
    A_MS = a.reshape((m,s))        #adjacency matrix for M/S connections
    
    for i in range(len(A_MS)):
        if np.sum(A_MS[i]) == 0:
            j = int(np.random.uniform(0,len(A_MS[i]),1))
            A_MS[i][j] = 1
    for i in range(len(A_MS.T)):
        if np.sum(A_MS.T[i]) == 0:
            j = int(np.random.uniform(0,len(A_MS.T[i]),1))
            A_MS[j][i] = 1
    print(A_MS)
            
    K_M = np.zeros((m,))+0.1      #maximum throughput of manufacturer
    K_M[0] = 0.13
    K_S = np.zeros((s,))+0.1       #maximum throughput of supplier (with optimal K_M)
    alpha_M = 0.01                     #internal perturbation of manufacturer
    alpha_S = 0.01                      #internal perturbation of supplier
    B_M1 = np.zeros((m,m))+1.0        #effects of price competition on manufacturer
    B_S1 = np.zeros((s,s))+4.0        #effects of price competition on supplier
    B_M2 = np.zeros((m,m))+0.5          #effects of technology competition on manufacturer
    B_S2 = np.zeros((s,s))+0.5          #effects of technology competition on supplier
    mu_M = np.zeros((m,))               #Manufacturer production outsourcing intensity
    mu_S = np.zeros((s,))               #Supplier production outsourcing intensity
    h = 2
    
    
    # package parameters and define time array
    parameters = [M, K_M, alpha_M, B_M1, B_M2, mu_M,S, K_S, alpha_S, B_S1, B_S2, mu_S, h, A_MS]
    fs, L = 200, 199
    t = np.linspace(0, L,int(L*fs))
    
    
    
    #run simulation
    M_arr, S_arr = MS_simulation(t, parameters)
    print(M_arr)
    A = MS_network(A_MS, M_arr, S_arr)
    
    
    #plot resulting simulation throughput
    for i in range(len(M_arr[0])):
        plt.plot(t, M_arr.T[i], label = r'$M_{'+str(i)+'}$')
    plt.legend()
    plt.ylim(0,)
    plt.grid()
    plt.show()
    
    for i in range(len(S_arr[0])):
        plt.plot(t, S_arr.T[i], label = r'$S_{'+str(i)+'}$')
    plt.legend()
    plt.ylim(0,)
    plt.grid()
    plt.show()

    
    
    
    





# -*- coding: utf-8 -*-
"""
Created on Fri Apr  9 14:36:56 2021

@author: myersau3
"""

import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="dynamical_networks", # Replace with your own username
    version="0.1.3",
    author="Audun Myers",
    author_email="myersau3@msu.edu",
    description="A package for dynamical networks simulations and analysis.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/myersau3/Dynamical_Networks",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)



# -*- coding: utf-8 -*-
"""
Created on Fri Mar  5 16:56:42 2021

@author: myersau3
"""
def degree_matrix(A):
    import numpy as np
    n = len(A[0]) #get degree of graph
    d = np.sum(A, axis = 1) #get degree vector from graph
    D = np.zeros((n,n)) #initialize degree matrix as zeros
    np.fill_diagonal(D, d) #fill diagonal with degree vector
    return D

def graph_laplacian(A):
    A = np.array(A)
    D = degree_matrix(A) #get degree matrix from adjacency matrix
    L = D - A #difference between degree matrix and adjacency matrix is graph laplacian
    return L

def lorenz(u1, u2, u3, parameters):
    rho, sigma, beta = parameters
    u1_dot = sigma*(u2 - u1)
    u2_dot = u1*(rho-u3) - u2
    u3_dot = u1*u2 - beta*u3
    return np.array([u1_dot, u2_dot, u3_dot])

def coupled_lorenz_system(t, A, Tau, U_0, parameters, eps):
    U = U_0
    N = len(A[0])
    import numpy as np
    dt = np.mean(np.diff(t))
    Us=[U]
    for t_i in range(len(t)):
        G_u = []
        for u in U:
            g_u_i = lorenz(u[0], u[1], u[2], parameters)
            G_u.append(g_u_i)
        G_u = np.array(G_u)
        dU = G_u + eps*np.matmul(A,U)
        U = U + dU*dt
        Us.append(U)
    Us = Us[:-1]
    return np.array(Us)
        
    
    
    

# In[ ]:


if __name__ == '__main__':
    import numpy as np
    import matplotlib.pyplot as plt
    #NxN adjacency matrix decribing influence between systems
    N = 5
    A = np.random.rand(N,N)-0.5
    A = A + A.T
    np.fill_diagonal(A,0)
    
    #nxn linking matrix
    n = 3
    Tau = np.zeros((n,n))
    Tau[1][0] = 1
    
    #Nxn initial conditions matrix
    U_0 = np.random.rand(N,n)-0.5
    
    #parameters for lorenz
    parameters = [60,10,8/3]\
    
    #set time of solution
    T = 400
    fs = 100
    t = np.linspace(0,T,fs*T)
    
    eps = 0.4
    
    U = coupled_lorenz_system(t, A, Tau, U_0, parameters, eps)

    xs = U.T[0]
    ys = U.T[1]
    zs = U.T[2]
    keep_time = 80
        
    
    for i in range(N):
        plt.plot(t[-fs*keep_time:], xs[i][-fs*keep_time:])
    plt.show()
            
            
            
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    